@@ -10,6 +10,9 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
+ *
+ *================================================================
+ * modified by fduncanh 2022
  */
 
 #include "mirror_buffer.h"
@@ -25,7 +28,6 @@
 #include <stdio.h>
 #include <inttypes.h>
 
-//#define DUMP_KEI_IV
 struct mirror_buffer_s {
     logger_t *logger;
     aes_ctx_t *aes_ctx;
@@ -47,8 +49,8 @@ mirror_buffer_init_aes(mirror_buffer_t *mirror_buffer, const uint64_t *streamCon
     /* AES key and IV */
     // Need secondary processing to use
     
-    sprintf((char*) aeskey_video, "AirPlayStreamKey%" PRIu64, *streamConnectionID);
-    sprintf((char*) aesiv_video, "AirPlayStreamIV%" PRIu64, *streamConnectionID);
+    snprintf((char*) aeskey_video, sizeof(aeskey_video), "AirPlayStreamKey%" PRIu64, *streamConnectionID);
+    snprintf((char*) aesiv_video, sizeof(aesiv_video), "AirPlayStreamIV%" PRIu64, *streamConnectionID);
 
     sha_ctx_t *ctx = sha_init();
     sha_update(ctx, aeskey_video, strlen((char*) aeskey_video));
@@ -63,13 +65,6 @@ mirror_buffer_init_aes(mirror_buffer_t *mirror_buffer, const uint64_t *streamCon
 
     // Need to be initialized externally
     mirror_buffer->aes_ctx = aes_ctr_init(aeskey_video, aesiv_video);
-
-#ifdef DUMP_KEI_IV
-    FILE* keyfile = fopen("/sdcard/111.keyiv", "wb");
-    fwrite(aeskey_video, 16, 1, keyfile);
-    fwrite(aesiv_video, 16, 1, keyfile);
-    fclose(keyfile);
-#endif
 }
 
 mirror_buffer_t *
